package com.sensu.automall.activity_search

import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.KeyEvent
import android.view.View
import com.annotation.IRouter
import com.qipeilong.base.commonadapter.rv.MultiItemTypeAdapter
import com.qpl.loadstate.NoDataLoadState
import com.qpl.loadstate.NoNetWorkLoadViewState
import com.sensu.automall.BaseActivity
import com.sensu.automall.R
import com.sensu.automall.URL
import com.sensu.automall.activity_main.MainActivity
import com.sensu.automall.adapter.SearchSortLeftAdapter
import com.sensu.automall.db.SearchSortDBUtils
import com.sensu.automall.fragment.SearchSortFragment
import com.sensu.automall.manager.SetStatusStrategry
import com.sensu.automall.manager.StatusStrategry
import com.sensu.automall.model.SearchSortItem
import com.sensu.automall.model.SearchSortModel
import com.sensu.automall.recyclerview.SearchSortHeadDecoration
import com.sensu.automall.recyclerview.TobeQuotePriceItemDecoration
import com.sensu.automall.utils.*
import io.reactivex.Observable
import kotlinx.android.synthetic.main.activity_search2.*
import org.json.JSONException
import org.json.JSONObject
import java.lang.Exception

@IRouter(path = "/SearchActivity",desc = "分类")
class SearchActivity2:BaseActivity{

    constructor(){
        activity_LayoutId= R.layout.activity_search2
    }

    lateinit var categoryAdapter: SearchSortLeftAdapter
    lateinit var searchSortFragment:SearchSortFragment
    val categoryList= arrayListOf<SearchSortModel>()
    val rightSorts = arrayListOf<SearchSortItem>()
    var isMoved=false
    var mLocalData =false
    var mLocalGsonStr =""

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        SetStatusStrategry(StatusStrategry.WHITE, this)
        cancelFullProgressView()
    }

    override fun initView() {
        super.initView()
        categoryAdapter=SearchSortLeftAdapter(this,R.layout.item_searchsort_left,categoryList)
        rv_sort.adapter=categoryAdapter
        rv_sort.addItemDecoration(TobeQuotePriceItemDecoration(this,TobeQuotePriceItemDecoration.VERTICAL_LIST,MassageUtils.dip2px(2f),R.color.bg_content))
        rv_sort.layoutManager=LinearLayoutManager(this)

        categoryAdapter.setOnItemClickListener(object:MultiItemTypeAdapter.OnItemClickListener{
            override fun onItemLongClick(view: View?, holder: RecyclerView.ViewHolder?, position: Int): Boolean {
                return false
            }

            override fun onItemClick(view: View?, holder: RecyclerView.ViewHolder?, position: Int) {
                if(searchSortFragment!=null){
                    isMoved=true
                    setChecked(position,true)
                }
            }
        })
        createFragment()
    }

    override fun loadData() {
        super.loadData()
        getDataFromDB()
    }

    fun getDataFromDB(){
        Observable.create<List<SearchSortModel>>{
            it.onNext(SearchSortDBUtils.instance.findAllFromSP(this@SearchActivity2))
        }.compose(RxjavaUtils.observableToMain()).subscribe({
            mLocalData =if(it?.size == 0)false else true
            categoryList.clear()
            categoryList.addAll(it)
            handleData()
            mLocalGsonStr =MassageUtils.getFromSP(this@SearchActivity2,
                    SharedPreferenceUtil.SEARCHSORT_TITLE,SharedPreferenceUtil.SEARCHSORT_KEY)
            GetViewCatalogPages()
        },{
            if (!MassageUtils.isNetworkConnected(this)) {
                mLoadState.showLoadViewState(NoNetWorkLoadViewState::class.java)
            }else{
                GetViewCatalogPages()
            }
        })
    }

    fun handleData(){
        rightSorts.clear()
        categoryList?.forEachIndexed { index, searchSortModel ->
            var nextParentTitle=""
            if(index < categoryList.size-1){
                nextParentTitle = categoryList.get(index+1).Title
            }
            val banner = SearchSortItem("",searchSortModel.Title,"",-1,-1,"",
                    true,false,false,false,searchSortModel.BannerItems,index.toString(),searchSortModel.Title,nextParentTitle)
            rightSorts.add(banner)

            searchSortModel.NavigationItems?.forEach {
                it.bannerType =false
                it.nav =true
                it.tag = index.toString()
                it.parentTitle =searchSortModel.Title
                it.nextParentTitle =nextParentTitle
                it.empty =false
                rightSorts.add(it)
            }

            var navigations = searchSortModel.NavigationItems?.size % SearchSortFragment.SPAN_COUNT

            var need_add_empty = SearchSortFragment.SPAN_COUNT - navigations
            if(navigations == 0)  need_add_empty=0

            var need_add_lastRow =navigations
            if(need_add_lastRow == 0){
                need_add_lastRow = SearchSortFragment.SPAN_COUNT
            }
            if(searchSortModel.NavigationItems?.size>0){
                for (i in 0 until need_add_lastRow){
                    searchSortModel.NavigationItems?.get(searchSortModel.NavigationItems?.size - i-1).lastrow =true
                }
            }
            for (i in 0 until need_add_empty){
                rightSorts.add(SearchSortItem("","","",-1,-1,"",
                        false,true,true,true, mutableListOf(),index.toString(),"",""))
            }

            searchSortModel.CatalogItems?.forEachIndexed { cata_index, searchSortItem ->
                searchSortItem.bannerType =false
                searchSortItem.nav =false
                searchSortItem.empty =false
                searchSortItem.tag = index.toString()
                searchSortItem.parentTitle =searchSortModel.Title
                searchSortItem.nextParentTitle =nextParentTitle
                rightSorts.add(searchSortItem)
            }
        }
        categoryAdapter.notifyDataSetChanged()

        if(searchSortFragment!=null ){
            searchSortFragment.setSorts(rightSorts)
        }
    }

    override fun reload() {
        super.reload()
        if(!mLocalData && MassageUtils.isNetworkConnected(this)){
            GetViewCatalogPages()
        }
    }

    fun GetViewCatalogPages(){
        if(!mLocalData){
            LoadingDialog.getInstance().ShowLoading(this,contentView,false)
        }
        val params =RequestParams()
        client.postRequest("GetViewCatalogPages",URL.HTTP_GetViewCatalogPages,params,activityKey)
    }

    // 关键字搜索
    fun searchKeywordClick(v: View) {
//        RequestBI("ChaZhaoPage_Search", "ChaZhaoPage_Search", "查找页_搜索", "")
        startActivity(Intent(this,SearchKeywordDialogActivity::class.java).putExtra("productlist",
                Constants.PRODUCT_LIST_JUMP_SEARCH))
    }

//    fun RequestBI(MoelType: String, Position: String, Title: String,
//                  uid: String) {
//        val requestParams = RequestParams()
//        requestParams.put("EventID", "ChaZhaoPage_Click")
//        requestParams.put("CurrentPage", "searchActivity")
//        requestParams.put("Reffer", "")
//        requestParams.put("UserIdentity", MassageUtils.getUUID(this))
//        requestParams.put("Platform", "android")
//        val json_obj = JSONObject()
//        try {
//            json_obj.put("MoelType", MoelType)
//            json_obj.put("Position", Position)
//            json_obj.put("Title", Title)
//            json_obj.put("UID", uid)
//            json_obj.put("userID", userID)
//        } catch (e: JSONException) {
//            e.printStackTrace()
//        }
//
//        requestParams.put("Data", json_obj.toString())
//        client.postRequest("BI", URL.HTTP_REQUESTTUHU, requestParams,
//                activityKey)
//
//    }

    //语音输入
    fun voiceRecognize(v:View){
        startActivityForResult(Intent(this, VoiceRecognizeActivity::class.java), 1000)
    }

    private fun setChecked( position:Int,isLeft:Boolean){
        if(isLeft){
            categoryAdapter.checkItemClick(position)
            var count =0
            for ( i in 0 until position){
                count +=(categoryList.get(i).CatalogItems?.size + categoryList.get(i).NavigationItems?.size)
                var navigations = categoryList[i].NavigationItems?.size % SearchSortFragment.SPAN_COUNT
                if(navigations > 0){
                    count +=(SearchSortFragment.SPAN_COUNT - navigations)
                }
            }

            count+=position
            searchSortFragment.leftClick(count)


        }else{
            if(isMoved){
                isMoved=false
            }else{
                categoryAdapter.checkItemClick(position)
            }
            SearchSortHeadDecoration.mCurrentTag=position.toString()
        }
        moveToCenter(position)
    }

    private fun moveToCenter(position: Int){
        var childAt = rv_sort.getChildAt(position - (rv_sort.layoutManager as LinearLayoutManager).findFirstVisibleItemPosition())
        childAt?.let {
            var y = it.top - rv_sort.height/2
            rv_sort.smoothScrollBy(0,y)
        }
    }

    private fun createFragment(){
        var fragmentTransaction =supportFragmentManager.beginTransaction()
        searchSortFragment= SearchSortFragment()
        var bundle =Bundle()
        bundle.putSerializable("category",rightSorts)
        searchSortFragment.arguments=bundle
        searchSortFragment.setCheckListener(object :SearchSortHeadDecoration.CheckListener{
            override fun check(position: Int, isScroll: Boolean) {
                setChecked(position,isScroll)
            }
        })
        if (supportFragmentManager.findFragmentByTag("searchsort")!=null){
            fragmentTransaction.show(supportFragmentManager.findFragmentByTag("searchsort")).commit()
        }else{
            fragmentTransaction.add(R.id.lin_fragment,searchSortFragment,"searchsort")
            fragmentTransaction.commit()
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if(requestCode == 1000){
            if (data != null) {
                val text = data.getStringExtra("text")
                val mode_stwitch = MassageUtils.getModeStwitch(this);
                if (TextUtils.isEmpty(mode_stwitch) || mode_stwitch == SharedPreferenceUtil.MODE_SWITCH_AutoMallProductList) {
                    val intent = Intent(this, AutoMallProductListActivity::class.java)
                    intent.putExtra("keyword",text)
                    startActivity(intent)
                } else if (mode_stwitch == SharedPreferenceUtil.MODE_SWITCH_OldProductList) {
                    OldProductListActivity.startActivity(this, "", "", "", "", text)
                }
            }
        }
    }

    override fun notifyDataChanged(`object`: JSONObject?) {
        super.notifyDataChanged(`object`)
        try {
            var body = `object`?.optJSONObject("body")
            var method = `object`?.optString("method")
            if("GetViewCatalogPages".equals(method)){
                var data = body?.optJSONArray("Data").toString()
                val searchsorts = GsonParseUtil.parseArray(data,SearchSortModel::class.java)

                if(searchsorts==null || searchsorts.size ==0){
                    if(!mLocalData){
                        mLoadState.showLoadViewState(NoDataLoadState::class.java)
                    }else{
                        //categoryList.clear()
                        handleData()
                        MassageUtils.deleteToSP(this,SharedPreferenceUtil.SEARCHSORT_TITLE)
                    }
                }else{
                    mLoadState.hideLoadViewState(NoDataLoadState::class.java)

                    if(mLocalGsonStr?.hashCode() === data?.hashCode()){
                    }else{
                        categoryList.clear()
                        categoryList.addAll(searchsorts)
                        handleData()
                        MassageUtils.saveToSP(this,SharedPreferenceUtil.SEARCHSORT_TITLE,SharedPreferenceUtil.SEARCHSORT_KEY,data)
                    }
                }
            }
        }catch (e:Exception){
            e.printStackTrace()
        }
    }

    override fun onKeyDown(keyCode: Int, event: KeyEvent?): Boolean {
        if ((keyCode == KeyEvent.KEYCODE_BACK && event?.action == KeyEvent.ACTION_DOWN))
        {
            if (parent is MainActivity) {
                (parent as MainActivity).tab(0)
            }
            return true
        }
        return super.onKeyDown(keyCode, event)
    }


}


