package com.sensu.automall.adapter

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.util.SparseArray
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.qipeilong.imageloader.ImageLoadManager
import com.sensu.automall.R
import com.sensu.automall.model.SearchSortItem
import com.sensu.automall.utils.MassageUtils
import com.yc.cn.ycbannerlib.banner.BannerView


class SearchSortAdapter : RecyclerView.Adapter<SearchSortAdapter.SearchSortHolder>{

    private val mList:List<SearchSortItem>
    private val mContext:Context
    private val mIndicators : SparseArray<LinearLayout> = SparseArray()

    constructor(context:Context,list:List<SearchSortItem>){
        mContext=context
        mList=list
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): SearchSortHolder {
        var contentView:View
        when(viewType){
            TITLE_TYPE->{
                contentView = LayoutInflater.from(mContext).inflate(R.layout.item_searchsort_title,parent,false)
            }
            CONTENT_TYPE->{
                contentView = LayoutInflater.from(mContext).inflate(R.layout.item_searchsort_right,parent,false)
            }
            NAVGATION_TYPE->{
                contentView = LayoutInflater.from(mContext).inflate(R.layout.item_searchsort_right_nav,parent,false)
            }

            else-> {
                contentView = LayoutInflater.from(mContext).inflate(R.layout.item_searchsort_right,parent,false)
            }
        }
        val viewHolder = SearchSortHolder(contentView,viewType,mContext,mIndicators)
        viewHolder.contentView.setOnClickListener {
            mItemClickListener?.onItemClick(viewHolder.adapterPosition,-1)
        }
        return viewHolder
    }

    override fun getItemCount(): Int {
        return mList?.size
    }

    override fun getItemViewType(position: Int): Int {
        if(mList?.get(position).bannerType){
            return TITLE_TYPE
        }else if (mList?.get(position).nav){
            return NAVGATION_TYPE
        }else{
            return CONTENT_TYPE
        }
    }

    override fun onBindViewHolder(holder: SearchSortHolder, position: Int) {
        holder.setData(position,mList[position])
    }


    inner class SearchSortHolder : RecyclerView.ViewHolder {

        lateinit var tv_title: TextView
        lateinit var tv_nav_content: TextView
        lateinit var iv_nav_content: ImageView
        lateinit var tv_content: TextView
        lateinit var iv_content: ImageView
        lateinit var vp_banner: BannerView
        lateinit var ll_indicator: LinearLayout
        lateinit var separtor: View
        val contentView: View
        val mContext: Context
        val mIndicators: SparseArray<LinearLayout>

        constructor(mContentView: View, type: Int, context: Context, indicators: SparseArray<LinearLayout>) : super(mContentView) {
            mContext = context
            mIndicators = indicators
            contentView = mContentView
            when (type) {
                TITLE_TYPE -> {
                    tv_title = mContentView.findViewById(R.id.tv_title)
                    vp_banner = mContentView.findViewById(R.id.vp_banner)
                    var layoutParam = vp_banner.layoutParams
                    layoutParam.height = MassageUtils.getSceenWidth() *3 /4 / 2

                    ll_indicator = mContentView.findViewById(R.id.ll_indicator)
                    ll_indicator.removeAllViews()
                }
                NAVGATION_TYPE -> {
                    tv_nav_content = mContentView.findViewById(R.id.tv_sort_nav_content)
                    iv_nav_content = mContentView.findViewById(R.id.iv_sort_nav_content)
                    separtor = mContentView.findViewById(R.id.separtor)
                }
                CONTENT_TYPE -> {
                    tv_content = mContentView.findViewById(R.id.tv_sort_content)
                    iv_content = mContentView.findViewById(R.id.iv_sort_content)
                }
            }
        }

        fun setData(position: Int, searchSortItem: SearchSortItem) {
            searchSortItem?.apply {
                if (bannerType) {
                    tv_title.text = Title
                    if (banner != null && banner.size > 0) {
                        vp_banner.visibility = View.VISIBLE
                        ll_indicator.visibility = View.VISIBLE
                        var imageUrls = arrayListOf<String>()
                        banner?.forEach {
                            imageUrls.add(it.ImageUrl)
                        }

                        //设置轮播时间
                        vp_banner.setPlayDelay(2000)
                        //设置轮播图适配器，必须
                        vp_banner.setAdapter(BannerImageAdapter(imageUrls))
                        //设置位置
                        vp_banner.setHintGravity(1)
                        //设置指示器样式
                        vp_banner.setHintMode(BannerView.HintMode.TEXT_HINT)
                        //判断轮播是否进行
                        val playing = vp_banner.isPlaying()
                        //轮播图点击事件
                        vp_banner.setOnBannerClickListener { itemPosition ->
                            mItemClickListener?.onItemClick(position, itemPosition)
                        }

                    } else {
                        vp_banner.visibility = View.GONE
                        ll_indicator.visibility = View.GONE
                    }
                } else if (nav) {
                    if (empty) {
                        tv_nav_content.text = ""
                        iv_nav_content.setImageDrawable(null)
                    } else {
                        tv_nav_content.text = Title
                        ImageLoadManager.instance.loadImage(mContext,ImageUrl, iv_nav_content)
                    }
                    if (lastrow) {
                        separtor.visibility = View.VISIBLE
                    } else {
                        separtor.visibility = View.GONE
                    }
                } else {
                    tv_content.text = Title
                    ImageLoadManager.instance.loadImage(mContext,ImageUrl, iv_content)
                }
            }
        }
    }




    companion object {
        val TITLE_TYPE =1
        val CONTENT_TYPE=2
        val NAVGATION_TYPE =3
    }

    interface OnItemClickListener{
        fun onItemClick(position:Int,itemPosition:Int)
    }

    private var mItemClickListener:OnItemClickListener ? =null

    fun  setOnItemClickListener(listener:OnItemClickListener){
        this.mItemClickListener = listener
    }
}