package com.sensu.automall.fragment

import android.os.Bundle
import android.support.v7.widget.GridLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.sensu.automall.BaseFragment
import com.sensu.automall.R
import com.sensu.automall.adapter.SearchSortAdapter
import com.sensu.automall.autotrack.AutoTrackEvent
import com.sensu.automall.autotrack.AutoTrackProperty
import com.sensu.automall.autotrack.AutoTrackUtil
import com.sensu.automall.model.SearchSortItem
import com.sensu.automall.recyclerview.SearchSortHeadDecoration
import com.sensu.automall.roter_serviceimpl.RouterMapping
import kotlinx.android.synthetic.main.fragment_searchsort.*
import org.json.JSONObject
import java.lang.Exception

class SearchSortFragment :BaseFragment() {

    var mCategorys = mutableListOf<SearchSortItem>()
    lateinit var mSubCategoryAdapter:SearchSortAdapter
    lateinit var mManager:GridLayoutManager
    var move=true
    var mIndex=0
    var mCheckListener:SearchSortHeadDecoration.CheckListener?=null

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        super.onCreateView(inflater, container, savedInstanceState)
        var contentView = inflater.inflate(R.layout.fragment_searchsort,container,false)
        return contentView
    }

    companion object {
        val SPAN_COUNT =3
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initView()
        initData()
    }

    private fun initView(){
        mSubCategoryAdapter = SearchSortAdapter(context!!,mCategorys)
        mManager=GridLayoutManager(context,SPAN_COUNT).also {
            it.spanSizeLookup=object :GridLayoutManager.SpanSizeLookup(){
                override fun getSpanSize(position: Int): Int {
                    return if(mCategorys?.get(position).bannerType){
                        SPAN_COUNT
                    }else  {
                        1
                    }
                }
            }
        }
        recyclerView.layoutManager=mManager
        var itemDecoration =SearchSortHeadDecoration(context!!,mCategorys)
        itemDecoration.setCheckListener(mCheckListener)
        recyclerView.addItemDecoration(itemDecoration)
        recyclerView.adapter=mSubCategoryAdapter
        recyclerView.addOnScrollListener(RecyclerViewListener())

        mSubCategoryAdapter.setOnItemClickListener(object :SearchSortAdapter.OnItemClickListener{
            override fun onItemClick(position: Int,itemPosition:Int) {
                if(position<0) return
                if(itemPosition>=0){ //banner点击
                    if(mCategorys[position].banner?.get(itemPosition).ContentKey?.length>0){
                        RouterMapping.start(activity,mCategorys[position].banner?.get(itemPosition).ContentKey)
                        try {
                            val properties = JSONObject()
                            properties.put(AutoTrackProperty.entranceName, AutoTrackProperty.categoryEntranceNameMap["banner"])
                            properties.put(AutoTrackProperty.entrancePosition, position)
                            properties.put(AutoTrackProperty.entranceContentType, mCategorys[position].banner.get(itemPosition).ContentType)
                            properties.put(AutoTrackProperty.entranceContentTitle, mCategorys[position].banner?.get(itemPosition).Title)
                            properties.put(AutoTrackProperty.entranceLinkUrl, mCategorys[position].banner?.get(itemPosition).ContentKey)
                            properties.put(AutoTrackProperty.entranceCategoryName, mCategorys[position].Title)
                            AutoTrackUtil.track_click(AutoTrackEvent.QPL_entranceCategory, properties)
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }
                }else{
                    if(!mCategorys[position].empty){
                        if(mCategorys[position].ContentKey?.length>0){
                            RouterMapping.start(activity,mCategorys[position].ContentKey)
                            try {
                                val properties = JSONObject()
                                properties.put(AutoTrackProperty.entranceName, AutoTrackProperty.categoryEntranceNameMap["banner"])
                                properties.put(AutoTrackProperty.entrancePosition, position)
                                properties.put(AutoTrackProperty.entranceContentType, mCategorys[position].banner.get(itemPosition).ContentType)
                                properties.put(AutoTrackProperty.entranceContentTitle, mCategorys[position].banner?.get(itemPosition).Title)
                                properties.put(AutoTrackProperty.entranceLinkUrl, mCategorys[position].banner?.get(itemPosition).ContentKey)
                                properties.put(AutoTrackProperty.entranceCategoryName, mCategorys[position].Title)
                                AutoTrackUtil.track_click(AutoTrackEvent.QPL_entranceCategory, properties)
                            } catch (e: Exception) {
                                e.printStackTrace()
                            }
                        }
                    }
                }
            }
        })
    }

    private fun initData(){
        try {
            val list:ArrayList<SearchSortItem> = arguments?.getSerializable("category") as ArrayList<SearchSortItem>
            mCategorys.addAll(list)
            mSubCategoryAdapter.notifyDataSetChanged()
        }catch (e:Exception){
            e.printStackTrace()
        }

    }

    fun setCheckListener(listener: SearchSortHeadDecoration.CheckListener){
        mCheckListener = listener
    }

    fun leftClick(position:Int){
        if(recyclerView ==null) return
        mIndex=position
        recyclerView.stopScroll()
        smoothMoveToPoisiton(position)
    }

    private fun smoothMoveToPoisiton(position:Int){
        var firstItem = mManager?.findFirstVisibleItemPosition()
        var lastItem =mManager?.findLastVisibleItemPosition()
        if(position<=firstItem){
            recyclerView.scrollToPosition(position)
        }else if(position<=lastItem){
            var top =recyclerView.getChildAt(position-firstItem).top
            recyclerView.scrollBy(0,top)
        }else{
            recyclerView.scrollToPosition(position)
            move=true
        }
    }

    fun setSorts(sorts:List<SearchSortItem>){
        mCategorys.clear()
        mCategorys.addAll(sorts)
        mSubCategoryAdapter.notifyDataSetChanged()
    }

    private inner class RecyclerViewListener : RecyclerView.OnScrollListener(){
        override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
            super.onScrollStateChanged(recyclerView, newState)
            if(move && newState ==RecyclerView.SCROLL_STATE_IDLE){
                move=false
                var n =mIndex -mManager.findFirstVisibleItemPosition()
                if(0<=n && n<recyclerView.childCount){
                    var top =recyclerView.getChildAt(n).top
                    recyclerView.smoothScrollBy(0,top)
                }
            }
        }

        override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
            super.onScrolled(recyclerView, dx, dy)
            if(move){
                move =false
                var n =mIndex -mManager.findFirstVisibleItemPosition()
                if(0<=n && n<recyclerView.childCount){
                    var top =recyclerView.getChildAt(n).top
                    recyclerView.scrollBy(0,top)
                }
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
    }
}