package com.sensu.automall.view

import android.animation.Animator
import android.animation.ValueAnimator
import android.content.Context
import android.content.res.TypedArray
import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.RectF
import android.util.AttributeSet
import android.view.View
import com.sensu.automall.R
import com.sensu.automall.utils.LogUtils
import com.sensu.automall.utils.MassageUtils

class CircleDownTimerView : View {

    var RADIUS = MassageUtils.dip2px(20f).toFloat()
    var ARC_RADIUS = MassageUtils.dip2px(2f).toFloat()
    val DEFAULT_WIDTH = MassageUtils.dip2px(44f)
    val DEFAULT_HEIGHT = MassageUtils.dip2px(44f)
    val CirclePaint:Paint
    val ContentPaint:Paint
    val CircleRoundPaint:Paint
    var mRectF :RectF ?=null
    val mValueAnimator:ValueAnimator
    val mContent:String
    val mDuration:Long
    var mArcRing=0f
    constructor(context:Context):this(context,null)
    constructor(context: Context,attrs:AttributeSet?):this(context,attrs,0)
    constructor(context: Context,attrs: AttributeSet?,defStyle:Int):super(context,attrs,defStyle){
        var ta :TypedArray = context.obtainStyledAttributes(attrs,R.styleable.CircleDownTimerView)
        mContent = ta.getString(R.styleable.CircleDownTimerView_circledowntimer_content)
        RADIUS= ta.getFloat(R.styleable.CircleDownTimerView_circledowntimer_radius,MassageUtils.dip2px(20f).toFloat())
        mDuration = ta.getInteger(R.styleable.CircleDownTimerView_circledowntimer_time,3).toLong()
        ta.recycle()

    }

    init {
        CirclePaint = Paint(Paint.ANTI_ALIAS_FLAG)
        CirclePaint.color = context.resources.getColor(R.color.hottext)
        CirclePaint.style = Paint.Style.FILL
        CirclePaint.alpha = (255 * 0.5).toInt()

        ContentPaint = Paint(Paint.ANTI_ALIAS_FLAG)
        ContentPaint.color = context.resources.getColor(R.color.white_text_color)
        ContentPaint.textSize = MassageUtils.dip2px(14f).toFloat()

        CircleRoundPaint = Paint(Paint.ANTI_ALIAS_FLAG)
        CircleRoundPaint.color = context.resources.getColor(R.color.white_text_color)
        CircleRoundPaint.style = Paint.Style.STROKE
        CircleRoundPaint.strokeWidth = ARC_RADIUS


        mValueAnimator = ValueAnimator.ofInt(-100,0)
    }

    fun startAnim(){
        mValueAnimator.duration = mDuration * 1000
        mValueAnimator.addUpdateListener { animation ->
            var value = animation.animatedValue as Int
            mArcRing =  (value /100f) * 360
            if(value>=0){
                animatorEndListener?.end()
            }
            invalidate()

        }
        mValueAnimator.start()
    }

    fun stopAnim(){
        mValueAnimator?.removeAllListeners()
        mValueAnimator?.cancel()
        removeAnimatorEndListener()
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        var widthMode = MeasureSpec.getMode(widthMeasureSpec)
        var widthSize = MeasureSpec.getSize(widthMeasureSpec)
        var heightMode = MeasureSpec.getMode(heightMeasureSpec)
        var heightSize = MeasureSpec.getSize(heightMeasureSpec)

        if (widthMode!=MeasureSpec.EXACTLY){
            widthSize = DEFAULT_WIDTH
        }
        if(heightMode != MeasureSpec.EXACTLY){
            heightSize = DEFAULT_HEIGHT
        }
        if(RADIUS > widthSize/2){
            RADIUS = widthSize/2f
        }
        setMeasuredDimension(widthSize,heightSize)
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        super.onLayout(changed, left, top, right, bottom)
        mRectF = RectF(ARC_RADIUS,ARC_RADIUS,measuredWidth.toFloat()-ARC_RADIUS,measuredHeight.toFloat()-ARC_RADIUS)
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        canvas?.drawCircle(measuredWidth/2.toFloat() ,measuredHeight/2.toFloat(),RADIUS,CirclePaint)
        canvas?.drawText(mContent,(measuredWidth-ContentPaint.measureText(mContent))/2,
                measuredHeight/2f + (ContentPaint.fontMetrics.bottom - ContentPaint.fontMetrics.top)/2 -
                        ContentPaint.fontMetrics.bottom,ContentPaint)
        canvas?.rotate(-90f,measuredWidth/2f,measuredHeight/2f)
        canvas?.drawArc(mRectF,-360f,mArcRing,false,CircleRoundPaint)
    }

    interface OnAnimatorEndListener{
        fun end()
    }
    private var animatorEndListener:OnAnimatorEndListener?=null

    fun setOnAnimatorEndListener(listener: OnAnimatorEndListener){
        animatorEndListener = listener
    }

    fun removeAnimatorEndListener(){
        animatorEndListener = null
    }
}