package com.sensu.automall.view

import android.content.Context
import android.content.res.TypedArray
import android.graphics.*
import android.graphics.drawable.BitmapDrawable
import android.util.AttributeSet
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import com.sensu.automall.R
import com.sensu.automall.utils.MassageUtils
import java.lang.RuntimeException

class StoreCertificateInfoView :View {

    var mWidth = MassageUtils.dip2px(290f)
    var mHeight = MassageUtils.dip2px(186f)

    private var mRectPaint:Paint?= null
    private var mPathPaint:Paint?=null
    private var mTextPaint:Paint?=null
    private var mCirclePaint:Paint?=null
    private var mBitmapPaint:Paint?=null
    private val pathLength = MassageUtils.dip2px(25f)
    private val sidePath = Path()
    private val padding = MassageUtils.dip2px(20f).toFloat()
    private val textMarginBottom = MassageUtils.dip2px(30f).toFloat()
    private val mRadius: Float
    private val mCamera_Src: BitmapDrawable
    private var mCamera_bitmap: Bitmap? = null
    private val mBg_Src:BitmapDrawable
    private var mBg_bitmap:Bitmap?=null

    private var text=""

    private var content = false

    constructor(context:Context):this(context,null)

    constructor(context: Context,attrs:AttributeSet?):this(context,attrs,0)

    constructor(context: Context,attrs:AttributeSet?,def:Int):super(context,attrs,def){

        val ta:TypedArray = context.obtainStyledAttributes(attrs, R.styleable.StoreCertificateInfoView)

        text = ta.getString(R.styleable.StoreCertificateInfoView_text)
        mRadius = ta.getDimension(R.styleable.StoreCertificateInfoView_info_radius, MassageUtils.dip2px(31f).toFloat())

        mCamera_Src = ta.getDrawable(R.styleable.StoreCertificateInfoView_info_camera) as BitmapDrawable
        mBg_Src = ta.getDrawable(R.styleable.StoreCertificateInfoView_bg_src) as BitmapDrawable
        mCamera_bitmap = mCamera_Src.bitmap
        mBg_bitmap = mBg_Src.bitmap
        if (mCamera_bitmap == null) {
            mCamera_bitmap = BitmapFactory.decodeResource(context.resources, R.drawable.storecertificate_camera)
        }
        if(mBg_bitmap == null){
            throw RuntimeException("please set bg image")
        }
        ta.recycle()


        init()
    }

    private fun init(){

        mRectPaint = Paint(Paint.ANTI_ALIAS_FLAG)
        mRectPaint?.color = Color.WHITE
       // mRectPaint?.setShadowLayer(50f,0f,0f,Color.parseColor("#1A000000"))
        //mRectPaint?.maskFilter = BlurMaskFilter(10f,BlurMaskFilter.Blur.NORMAL)

        mPathPaint = Paint(Paint.ANTI_ALIAS_FLAG)
        mPathPaint?.style = Paint.Style.STROKE
        mPathPaint?.strokeWidth = MassageUtils.dip2px(3f).toFloat()
        mPathPaint?.color = Color.parseColor("#fffac3c6")

        mTextPaint = Paint(Paint.ANTI_ALIAS_FLAG)
        mTextPaint?.color = context.resources.getColor(R.color.gray_vin)
        mTextPaint?.textSize = MassageUtils.dip2px(16f).toFloat()


        mCirclePaint = Paint(Paint.ANTI_ALIAS_FLAG)
        var linearGradient = LinearGradient(mWidth / 2 - mRadius, mHeight / 2 - mRadius, mWidth / 2 - mRadius, mHeight / 2 + mRadius,
                Color.parseColor("#FF8359"), Color.parseColor("#F03744"), Shader.TileMode.MIRROR)
        mCirclePaint?.style = Paint.Style.FILL
        mCirclePaint?.shader = linearGradient

        mBitmapPaint = Paint(Paint.ANTI_ALIAS_FLAG)
        //setWillNotDraw(false)
        //setLayerType(View.LAYER_TYPE_SOFTWARE, null)
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)

        var widthMode = MeasureSpec.getMode(widthMeasureSpec)
        var widthSize = MeasureSpec.getSize(widthMeasureSpec)

        var heightMode = MeasureSpec.getMode(heightMeasureSpec)
        var heightSize = MeasureSpec.getSize(heightMeasureSpec)

        if(widthMode!=MeasureSpec.EXACTLY){
            widthSize = mWidth
        }else{
            mWidth = widthSize
        }
        if(heightMode!=MeasureSpec.EXACTLY){
            heightSize = mHeight
        }else{
            mHeight = heightSize
        }

        setMeasuredDimension(widthSize,heightSize)
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)

        canvas?.drawRoundRect(RectF(10f,10f,mWidth-10f,mHeight-10f),5f,5f,mRectPaint)


        sidePath.moveTo(padding,padding+pathLength)
        sidePath.lineTo(padding,padding)
        sidePath.lineTo(padding+pathLength,padding)
        canvas?.drawPath(sidePath,mPathPaint)

        sidePath.moveTo(mWidth-padding-pathLength,padding)
        sidePath.lineTo(mWidth-padding,padding)
        sidePath.lineTo(mWidth-padding,padding+pathLength)

        canvas?.drawPath(sidePath,mPathPaint)


        sidePath.moveTo(mWidth-padding,mHeight-padding-pathLength)
        sidePath.lineTo(mWidth-padding,mHeight-padding)
        sidePath.lineTo(mWidth-padding-pathLength,mHeight-padding)

        canvas?.drawPath(sidePath,mPathPaint)

        sidePath.moveTo(padding+pathLength,mHeight-padding)
        sidePath.lineTo(padding,mHeight-padding)
        sidePath.lineTo(padding,mHeight-padding-pathLength)

        canvas?.drawPath(sidePath,mPathPaint)

        if(!content){
            canvas?.drawText(text,(mWidth - mTextPaint?.measureText(text)!!)/2,mHeight- textMarginBottom,mTextPaint)

            canvas?.drawBitmap(mBg_bitmap, (mWidth - mBg_bitmap?.width!!) / 2f, (mHeight- mBg_bitmap?.height!!)/ 2f, mBitmapPaint)

            canvas?.drawCircle((mWidth / 2).toFloat(), (mHeight / 2).toFloat(), mRadius, mCirclePaint)

            canvas?.drawBitmap(mCamera_bitmap, (mWidth - mCamera_bitmap?.width!!) / 2f, (mHeight- mCamera_bitmap?.height!!)/ 2f, mBitmapPaint)
        }
    }

    fun setContent(content:Boolean){
        this.content = content
        invalidate()
    }


}